import { useNavigate, useParams } from "@solidjs/router";
import { Component, Show, createEffect, createSignal, onMount } from "solid-js";

import styles from './GroupPage.module.css'
import { Group, Map, Profile, Strat, User, api } from "../api";
import { showDialog, showInputDialog, showMessageDialog } from "../state";
import { _arrayBufferToBase64, addMap, addMember, addProfile, addStratType, errorToString, normalize } from "../util";
import { BsCheck2, BsPencil, BsTrash } from "solid-icons/bs";
import { Collapse } from "solid-collapse";
import MemberItem from "../components/MemberItem";
import MapItem from "../components/MapItem";
import StratItem from "../components/StratItem";
import StratTypeItem from "../components/StratTypeItem";
import ProfileItem from "../components/ProfileItem";
import { UploadFile, createFileUploader } from "@solid-primitives/upload";

const GroupPage: Component = () => {
	const params = useParams();
	const navigate = useNavigate();

	const [loading, setLoading] = createSignal(true);
	const [group, setGroup] = createSignal({} as Group);
	const [editing, setEditing] = createSignal(false);
	const [name, setName] = createSignal('');

	const [membersIsExpanded, setMembersIsExpanded] = createSignal(false);
	const [mapsIsExpanded, setMapsIsExpanded] = createSignal(false);
	const [stratsIsExpanded, setStratsIsExpanded] = createSignal(false);
	const [profilesIsExpanded, setProfilesIsExpanded] = createSignal(false);
	const [stratTypesIsExpanded, setStratTypesIsExpanded] = createSignal(false);

	onMount(async () => {
		try {
			setGroup(await api().getGroup(params.id));
			setName(group().name);
			setLoading(false);
		} catch (e) {
			showMessageDialog('Failed to load group', errorToString(e));
		}
	});

	const updateGroup = async (newValue: Group) => {
		try {
			await api().updateGroup(group().id, newValue);
			setGroup(newValue);
		} catch (e) {
			showMessageDialog('Failed to update group', errorToString(e));
		}
	};

	const deleteGroup = async () => {
		showDialog({
			title: 'Delete group',
			text: 'Do you want to delete this group?',
			buttons: [
				{
					name: 'Delete',
					type: 'danger',
					action: async () => {
						try {
							await api().removeGroup(group().id);
							navigate('/groups');
						} catch (e) {
							showMessageDialog('Failed to delete group', errorToString(e));
						}
					}
				},
				{
					name: 'Cancel',
					action: () => { }
				}
			]
		});
	};

	const createStrat = async () => {
		const [title, setTitle] = createSignal('');
		const [description, setDescription] = createSignal('');
		const [stratType, setStratType] = createSignal('');

		showDialog({
			title: 'Add a strat to Group',
			text: 'create a new strat',
			inputFields: [{
				onInput: setTitle,
				placeholder: 'title',
				name: 'title'
			}, {
				onInput: setDescription,
				placeholder: 'description',
				name: 'description'
			}],
			onDismiss: async () => {
				const strat = await api().addStrat(group().id, { title: title(), description: description(), stratType: stratType() });
				navigate('/group/' + group().id + '/strat/' + strat.id);
			},
			content: <select name="stratType" id="stratType">
				{group().stratTypes.map(p => <option value={p} onclick={() => setStratType(p)}>{p}</option>)}
			</select>,
			buttons: ['confirm'],

		});
	}

	const toggleEdit = async () => {
		if (!editing()) {
			setEditing(true);
		} else {
			console.log(name());
			updateGroup({ ...group(), name: name() });
			setEditing(false);
		}
	};

	const onDeleteMember = async (member: User) => {
		setGroup({ ...group(), members: [...group().members.filter(m => m.username !== member.username)] });
	};

	const onDeleteMap = async (map: Map) => {
		setGroup({ ...group(), maps: [...group().maps.filter(m => m.id !== map.id)] });
	};

	const onDeleteStrat = async (strat: Strat) => {
		setGroup({ ...group(), strats: [...group().strats.filter(s => s.id !== strat.id)] });
	};

	const onDeleteStratType = async (stratType: string) => {
		setGroup({ ...group(), stratTypes: [...group().stratTypes.filter(s => s !== stratType)] });
	};

	const onDeleteProfile = async (profile: Profile) => {
		setGroup({ ...group(), profiles: [...group().profiles.filter(p => p.id !== profile.id)] });
	};

	return (
		<>
			<Show when={!loading()} fallback={<div>Loading...</div>}>
				<div class={styles.groupName}>
					<span contentEditable={editing()} innerText={group().name} onInput={e => setName(normalize(e.currentTarget.innerText, 50, true))} />
					{editing() && <span class={styles.editCount}>{name().length}/50</span>}
				</div>
				<div class={styles.groupButtons}>
					<button class={styles.deleteButton} onClick={deleteGroup}>Delete group</button>
					<button class={styles.editButton} onClick={toggleEdit}>{editing() ? <BsCheck2 /> : <BsPencil />}</button>
				</div>
				<hr />
				<div>
					<button onClick={() => setMembersIsExpanded(!membersIsExpanded())}>Members</button>
					<button class={styles.addMemberButton} onClick={async () => addMember(group(), (g) => setGroup(g))}>+</button>
					<Collapse value={membersIsExpanded()} class={styles.members}>
						<div class={styles.groupMembers}>
							{group().members?.map(m => <MemberItem group={group()} member={m} onDelete={onDeleteMember} />)}
						</div>
					</Collapse>
					<br />
					<button onClick={() => setStratsIsExpanded(!stratsIsExpanded())}>strats</button>
					<button class={styles.addStratButton} onClick={() => createStrat()}>+</button>
					<Collapse value={stratsIsExpanded()} class={styles.strats}>
						<div class={styles.strats}>
							{group().strats?.map(s => <StratItem group={group()} strat={s} onDelete={onDeleteStrat} />)}
						</div>
					</Collapse>
					<br />
					<button onClick={() => setMapsIsExpanded(!mapsIsExpanded())}>maps</button>
					<button class={styles.addMapButton} onClick={async () => addMap(group(), (g) => setGroup(g))}>+</button>
					<Collapse value={mapsIsExpanded()} class={styles.maps}>
						<div class={styles.maps}>
							{group().maps?.map(m => <MapItem group={group()} map={m} onDelete={onDeleteMap} />)}
						</div>
					</Collapse>
					<br />
					<button onClick={() => setStratTypesIsExpanded(!stratTypesIsExpanded())}>strat types</button>
					<button class={styles.addMemberButton} onClick={async () => addStratType(group(), (g) => setGroup(g))}>+</button>
					<Collapse value={stratTypesIsExpanded()} class={styles.stratTypes}>
						<div class={styles.stratTypes}>
							{group().stratTypes?.map(s => <StratTypeItem group={group()} stratType={s} onDelete={onDeleteStratType} />)}
						</div>
					</Collapse>
					<br />
					<button onClick={() => setProfilesIsExpanded(!profilesIsExpanded())}>profiles</button>
					<button class={styles.addMemberButton} onClick={async () => addProfile(group(), (g) => setGroup(g))}>+</button>
					<Collapse value={profilesIsExpanded()} class={styles.profiles}>
						<div class={styles.profiles}>
							{group().profiles?.map(p => <ProfileItem group={group()} profile={p} onDelete={onDeleteProfile} />)}
						</div>
					</Collapse>
				</div>
			</Show>
		</>
	)
}

export default GroupPage;
